import java.util.*;
import java.io.IOException;
import java.util.logging.Logger;

public class MyBot extends Bot {
    /**
     * Main method executed by the game engine for starting the bot.
     * 
     * @param args command line arguments
     * 
     * @throws IOException if an I/O error occurs
     */
    public static void main(String[] args) throws IOException {
        new MyBot().readSystemInput();
    }
	
	private Map<Tile, Tile> orders = new HashMap<Tile, Tile>(); // store orders issued as destination tile, origin tile
	private Set<Tile> rememberedFood = new HashSet<Tile>();
	private Set<Integer> enemyPlayersKnown = new HashSet<Integer>();
	private int passableGrid[][];
	private int foodAntGrid[][];
	private int combatStrengthGrid[][][];
	private int totalCombatStrengthGrid[][];
	private double scentGrid[][];
	private double enemyGrid[][];
	private int fighting[][][];
	private int status[][][];
	private int rows;
	private int cols;
	private Set<Tile> attackRange;
	private Set<Tile> northAttackRange;
	private Set<Tile> westAttackRange;
	private Set<Tile> eastAttackRange;
	private Set<Tile> southAttackRange;
	
	private final int SCENTGRID_DIFFUSION_TIMES = 100;
	private final int ENEMYGRID_DIFFUSION_TIMES = 50;
	private final int RESERVE_TIME = 100;
	private final double DIFFUSION_RATE = 0.20;
	private final double FOOD_VALUE = 1000;
	private final double EXPLORE_VALUE = 1000;
	private final double ENEMY_HILL_VALUE = 10000;
	private final double ENEMY_ANT_BONUS = 1;
	private final double DEAD_ANT_BONUS = 1;
	private final double UNSEEN_BONUS = 100;
	
	private int findStrongestEnemy(Tile myAnt) {
		int thisRow = myAnt.getRow();
		int thisCol = myAnt.getCol();
		int minEnemyWeakness = Integer.MAX_VALUE;
		for (Integer nextEnemy : enemyPlayersKnown) {
			int northernmost1Fighting = fighting[nextEnemy][rowOffset(thisRow, -2)][colOffset(thisCol, -1)];
			if (northernmost1Fighting > 0 && northernmost1Fighting < minEnemyWeakness) {
				minEnemyWeakness = northernmost1Fighting;
			}
			int northernmost2Fighting = fighting[nextEnemy][rowOffset(thisRow, -2)][thisCol];
			if (northernmost2Fighting > 0 && northernmost2Fighting < minEnemyWeakness) {
				minEnemyWeakness = northernmost2Fighting;
			}
			int northernmost3Fighting = fighting[nextEnemy][rowOffset(thisRow, -2)][colOffset(thisCol, 1)];
			if (northernmost3Fighting > 0 && northernmost3Fighting < minEnemyWeakness) {
				minEnemyWeakness = northernmost3Fighting;
			}
			int north1Fighting = fighting[nextEnemy][rowOffset(thisRow, -1)][colOffset(thisCol, -2)];
			if (north1Fighting > 0 && north1Fighting < minEnemyWeakness) {
				minEnemyWeakness = north1Fighting;
			}
			int north2Fighting = fighting[nextEnemy][rowOffset(thisRow, -1)][colOffset(thisCol, -1)];
			if (north2Fighting > 0 && north2Fighting < minEnemyWeakness) {
				minEnemyWeakness = north2Fighting;
			}
			int north3Fighting = fighting[nextEnemy][rowOffset(thisRow, -1)][thisCol];
			if (north3Fighting > 0 && north3Fighting < minEnemyWeakness) {
				minEnemyWeakness = north3Fighting;
			}
			int north4Fighting = fighting[nextEnemy][rowOffset(thisRow, -1)][colOffset(thisCol, 1)];
			if (north4Fighting > 0 && north4Fighting < minEnemyWeakness) {
				minEnemyWeakness = north4Fighting;
			}
			int north5Fighting = fighting[nextEnemy][rowOffset(thisRow, -1)][colOffset(thisCol, 2)];
			if (north5Fighting > 0 && north5Fighting < minEnemyWeakness) {
				minEnemyWeakness = north5Fighting;
			}
			int west1Fighting = fighting[nextEnemy][thisRow][colOffset(thisCol, -2)];
			if (west1Fighting > 0 && west1Fighting < minEnemyWeakness) {
				minEnemyWeakness = west1Fighting;
			}
			int west2Fighting = fighting[nextEnemy][thisRow][colOffset(thisCol, -1)];
			if (west2Fighting > 0 && west2Fighting < minEnemyWeakness) {
				minEnemyWeakness = west2Fighting;
			}
			int east1Fighting = fighting[nextEnemy][thisRow][colOffset(thisCol, 1)];
			if (east1Fighting > 0 && east1Fighting < minEnemyWeakness) {
				minEnemyWeakness = east1Fighting;
			}
			int east2Fighting = fighting[nextEnemy][thisRow][colOffset(thisCol, 2)];
			if (east2Fighting > 0 && east2Fighting < minEnemyWeakness) {
				minEnemyWeakness = east2Fighting;
			}
			int south1Fighting = fighting[nextEnemy][rowOffset(thisRow, 1)][colOffset(thisCol, -2)];
			if (south1Fighting > 0 && south1Fighting < minEnemyWeakness) {
				minEnemyWeakness = south1Fighting;
			}
			int south2Fighting = fighting[nextEnemy][rowOffset(thisRow, 1)][colOffset(thisCol, -1)];
			if (south2Fighting > 0 && south2Fighting < minEnemyWeakness) {
				minEnemyWeakness = south2Fighting;
			}
			int south3Fighting = fighting[nextEnemy][rowOffset(thisRow, 1)][thisCol];
			if (south3Fighting > 0 && south3Fighting < minEnemyWeakness) {
				minEnemyWeakness = south3Fighting;
			}
			int south4Fighting = fighting[nextEnemy][rowOffset(thisRow, 1)][colOffset(thisCol, 1)];
			if (south4Fighting > 0 && south4Fighting < minEnemyWeakness) {
				minEnemyWeakness = south4Fighting;
			}
			int south5Fighting = fighting[nextEnemy][rowOffset(thisRow, 1)][colOffset(thisCol, 2)];
			if (south5Fighting > 0 && south5Fighting < minEnemyWeakness) {
				minEnemyWeakness = south5Fighting;
			}
			int southernmost1Fighting = fighting[nextEnemy][rowOffset(thisRow, 2)][colOffset(thisCol, -1)];
			if (southernmost1Fighting > 0 && southernmost1Fighting < minEnemyWeakness) {
				minEnemyWeakness = southernmost1Fighting;
			}
			int southernmost2Fighting = fighting[nextEnemy][rowOffset(thisRow, 2)][thisCol];
			if (southernmost2Fighting > 0 && southernmost2Fighting < minEnemyWeakness) {
				minEnemyWeakness = southernmost2Fighting;
			}
			int southernmost3Fighting = fighting[nextEnemy][rowOffset(thisRow, 2)][colOffset(thisCol, 1)];
			if (southernmost3Fighting > 0 && southernmost3Fighting < minEnemyWeakness) {
				minEnemyWeakness = southernmost3Fighting;
			}
		}
		return minEnemyWeakness;
	}
	
	private void fillStatusGrid(Map<Tile, Integer> myAnts) {
		for (Tile nextAnt : myAnts.keySet()) {
			if (myAnts.get(nextAnt) == 1) {
				continue;
			}
			int thisRow = nextAnt.getRow();
			int thisCol = nextAnt.getCol();
			int enemyCombatStrength = totalCombatStrengthGrid[thisRow][thisCol] - combatStrengthGrid[0][thisRow][thisCol];
			int minEnemyWeakness = findStrongestEnemy(nextAnt);
			if (enemyCombatStrength > 0 && enemyCombatStrength < minEnemyWeakness) {
				status[0][thisRow][thisCol] = 2;
			}
			else if (enemyCombatStrength < minEnemyWeakness) {
				status[0][thisRow][thisCol] = 1;
			}
			else {
				status[0][thisRow][thisCol] = 0;
			}
			if (passableGrid[rowOffset(thisRow, -1)][thisCol] == 1) {
				int northRow = rowOffset(thisRow, -1);
				int northCol = thisCol;
				Tile northAnt = new Tile(northRow, northCol);
				enemyCombatStrength = totalCombatStrengthGrid[northRow][northCol] - combatStrengthGrid[0][northRow][northCol];
				minEnemyWeakness = findStrongestEnemy(northAnt);
				if (enemyCombatStrength > 0 && enemyCombatStrength < minEnemyWeakness) {
					status[0][northRow][northCol] = 2;
				}
				else if (enemyCombatStrength < minEnemyWeakness) {
					status[0][northRow][northCol] = 1;
				}
				else {
					status[0][northRow][northCol] = 0;
				}
			}
			if (passableGrid[thisRow][colOffset(thisCol, -1)]== 1) {
				int westRow = thisRow;
				int westCol = colOffset(thisCol, -1);
				Tile westAnt = new Tile(westRow, westCol);
				enemyCombatStrength = totalCombatStrengthGrid[westRow][westCol] - combatStrengthGrid[0][westRow][westCol];
				minEnemyWeakness = findStrongestEnemy(westAnt);
				if (enemyCombatStrength > 0 && enemyCombatStrength < minEnemyWeakness) {
					status[0][westRow][westCol] = 2;
				}
				else if (enemyCombatStrength < minEnemyWeakness) {
					status[0][westRow][westCol] = 1;
				}
				else {
					status[0][westRow][westCol] = 0;
				}
			}
			if (passableGrid[thisRow][colOffset(thisCol, 1)]== 1) {
				int eastRow = thisRow;
				int eastCol = colOffset(thisCol, 1);
				Tile eastAnt = new Tile(eastRow, eastCol);
				enemyCombatStrength = totalCombatStrengthGrid[eastRow][eastCol] - combatStrengthGrid[0][eastRow][eastCol];
				minEnemyWeakness = findStrongestEnemy(eastAnt);
				if (enemyCombatStrength > 0 && enemyCombatStrength < minEnemyWeakness) {
					status[0][eastRow][eastCol] = 2;
				}
				else if (enemyCombatStrength < minEnemyWeakness) {
					status[0][eastRow][eastCol] = 1;
				}
				else {
					status[0][eastRow][eastCol] = 0;
				}
			}
			if (passableGrid[rowOffset(thisRow, 1)][thisCol]== 1) {
				int southRow = rowOffset(thisRow, 1);
				int southCol = thisCol;
				Tile southAnt = new Tile(southRow, southCol);
				enemyCombatStrength = totalCombatStrengthGrid[southRow][southCol] - combatStrengthGrid[0][southRow][southCol];
				minEnemyWeakness = findStrongestEnemy(southAnt);
				if (enemyCombatStrength > 0 && enemyCombatStrength < minEnemyWeakness) {
					status[0][southRow][southCol] = 2;
				}
				else if (enemyCombatStrength < minEnemyWeakness) {
					status[0][southRow][southCol] = 1;
				}
				else {
					status[0][southRow][southCol] = 0;
				}
			}
		}
	}
	
	private void fillFightingGrid(Ants ants, Map<Tile, Integer> myAnts, Set<Tile> enemyAnts) {
		/*
		for (Tile nextAnt : myAnts.keySet()) {
			int thisRow = nextAnt.getRow();
			int thisCol = nextAnt.getCol();
			int alreadyMoved = myAnts.get(nextAnt);
			fighting[0][thisRow][thisCol] = totalCombatStrengthGrid[thisRow][thisCol] - combatStrengthGrid[0][thisRow][thisCol];
			if (alreadyMoved == 1) {
				continue;
			}
			else {
				if (passableGrid[rowOffset(thisRow, -1)][thisCol] == 1) {
					fighting[0][rowOffset(thisRow, -1)][thisCol] = totalCombatStrengthGrid[rowOffset(thisRow, -1)][thisCol] - combatStrengthGrid[0][rowOffset(thisRow, -1)][thisCol];
				}
				if (passableGrid[thisRow][colOffset(thisCol, -1)] == 1) {
					fighting[0][thisRow][colOffset(thisCol, -1)] = totalCombatStrengthGrid[thisRow][colOffset(thisCol, -1)] - combatStrengthGrid[0][thisRow][colOffset(thisCol, -1)];
				}
				if (passableGrid[thisRow][colOffset(thisCol, 1)] == 1) {
					fighting[0][thisRow][colOffset(thisCol, 1)] = totalCombatStrengthGrid[thisRow][colOffset(thisCol, 1)] - combatStrengthGrid[0][thisRow][colOffset(thisCol, 1)];
				}
				if (passableGrid[rowOffset(thisRow, 1)][thisCol] == 1) {
					fighting[0][rowOffset(thisRow, 1)][thisCol] = totalCombatStrengthGrid[rowOffset(thisRow, 1)][thisCol] - combatStrengthGrid[0][rowOffset(thisRow, 1)][thisCol];
				}
			}
		}
		*/
		for (Tile nextAnt : enemyAnts) {
			int owner = ants.getEnemyAntOwner(nextAnt);
			int thisRow = nextAnt.getRow();
			int thisCol = nextAnt.getCol();
			fighting[owner][thisRow][thisCol] = totalCombatStrengthGrid[thisRow][thisCol] - combatStrengthGrid[owner][thisRow][thisCol];
			if (passableGrid[rowOffset(thisRow, -1)][thisCol] == 1) {
				fighting[owner][rowOffset(thisRow, -1)][thisCol] = totalCombatStrengthGrid[rowOffset(thisRow, -1)][thisCol] - combatStrengthGrid[owner][rowOffset(thisRow, -1)][thisCol];
			}
			if (passableGrid[thisRow][colOffset(thisCol, -1)] == 1) {
				fighting[owner][thisRow][colOffset(thisCol, -1)] = totalCombatStrengthGrid[thisRow][colOffset(thisCol, -1)] - combatStrengthGrid[owner][thisRow][colOffset(thisCol, -1)];
			}
			if (passableGrid[thisRow][colOffset(thisCol, 1)] == 1) {
				fighting[owner][thisRow][colOffset(thisCol, 1)] = totalCombatStrengthGrid[thisRow][colOffset(thisCol, 1)] - combatStrengthGrid[owner][thisRow][colOffset(thisCol, 1)];
			}
			if (passableGrid[rowOffset(thisRow, 1)][thisCol] == 1) {
				fighting[owner][rowOffset(thisRow, 1)][thisCol] = totalCombatStrengthGrid[rowOffset(thisRow, 1)][thisCol] - combatStrengthGrid[owner][rowOffset(thisRow, 1)][thisCol];
			}
		}	
	}
	
	private void refillCombatGrid(int owner, Tile oldTile, Tile newTile) {
		int oldRow = oldTile.getRow();
		int oldCol = oldTile.getCol();
		combatStrengthGrid[owner][oldRow][oldCol]--;											// own tile
		totalCombatStrengthGrid[oldRow][oldCol]--;
		combatStrengthGrid[owner][rowOffset(oldRow, -2)][colOffset(oldCol, -1)]--;				// first northernmost tile
		totalCombatStrengthGrid[rowOffset(oldRow, -2)][colOffset(oldCol, -1)]--;
		combatStrengthGrid[owner][rowOffset(oldRow, -2)][oldCol]--;								// second northernmost tile
		totalCombatStrengthGrid[rowOffset(oldRow, -2)][oldCol]--;
		combatStrengthGrid[owner][rowOffset(oldRow, -2)][colOffset(oldCol, 1)]--;				// third northernmost tile
		totalCombatStrengthGrid[rowOffset(oldRow, -2)][colOffset(oldCol, 1)]--;
		combatStrengthGrid[owner][rowOffset(oldRow, -1)][colOffset(oldCol, -2)]--;				// first north tile
		totalCombatStrengthGrid[rowOffset(oldRow, -1)][colOffset(oldCol, -2)]--;
		combatStrengthGrid[owner][rowOffset(oldRow, -1)][colOffset(oldCol, -1)]--;				// second north tile
		totalCombatStrengthGrid[rowOffset(oldRow, -1)][colOffset(oldCol, -1)]--;
		combatStrengthGrid[owner][rowOffset(oldRow, -1)][oldCol]--;								// third north tile
		totalCombatStrengthGrid[rowOffset(oldRow, -1)][oldCol]--;
		combatStrengthGrid[owner][rowOffset(oldRow, -1)][colOffset(oldCol, 1)]--;				// fourth north tile
		totalCombatStrengthGrid[rowOffset(oldRow, -1)][colOffset(oldCol, 1)]--;
		combatStrengthGrid[owner][rowOffset(oldRow, -1)][colOffset(oldCol, 2)]--;				// fifth north tile
		totalCombatStrengthGrid[rowOffset(oldRow, -1)][colOffset(oldCol, 2)]--;
		combatStrengthGrid[owner][oldRow][colOffset(oldCol, -2)]--;								// first west tile
		totalCombatStrengthGrid[oldRow][colOffset(oldCol, -2)]--;
		combatStrengthGrid[owner][oldRow][colOffset(oldCol, -1)]--;								// second west tile
		totalCombatStrengthGrid[oldRow][colOffset(oldCol, -1)]--;
		combatStrengthGrid[owner][oldRow][colOffset(oldCol, 1)]--;								// first east tile
		totalCombatStrengthGrid[oldRow][colOffset(oldCol, 1)]--;
		combatStrengthGrid[owner][oldRow][colOffset(oldCol, 2)]--;								// second east tile
		totalCombatStrengthGrid[oldRow][colOffset(oldCol, 2)]--;
		combatStrengthGrid[owner][rowOffset(oldRow, 1)][colOffset(oldCol, -2)]--;				// first south tile
		totalCombatStrengthGrid[rowOffset(oldRow, 1)][colOffset(oldCol, -2)]--;
		combatStrengthGrid[owner][rowOffset(oldRow, 1)][colOffset(oldCol, -1)]--;				// second south tile
		totalCombatStrengthGrid[rowOffset(oldRow, 1)][colOffset(oldCol, -1)]--;
		combatStrengthGrid[owner][rowOffset(oldRow, 1)][oldCol]--;								// third south tile
		totalCombatStrengthGrid[rowOffset(oldRow, 1)][oldCol]--;
		combatStrengthGrid[owner][rowOffset(oldRow, 1)][colOffset(oldCol, 1)]--;				// fourth south tile
		totalCombatStrengthGrid[rowOffset(oldRow, 1)][colOffset(oldCol, 1)]--;
		combatStrengthGrid[owner][rowOffset(oldRow, 1)][colOffset(oldCol, 2)]--;				// fifth south tile
		totalCombatStrengthGrid[rowOffset(oldRow, 1)][colOffset(oldCol, 2)]--;
		combatStrengthGrid[owner][rowOffset(oldRow, 2)][colOffset(oldCol, -1)]--;				// first southernmost tile
		totalCombatStrengthGrid[rowOffset(oldRow, 2)][colOffset(oldCol, -1)]--;
		combatStrengthGrid[owner][rowOffset(oldRow, 2)][oldCol]--;								// second southernmost tile
		totalCombatStrengthGrid[rowOffset(oldRow, 2)][oldCol]--;
		combatStrengthGrid[owner][rowOffset(oldRow, 2)][colOffset(oldCol, 1)]--;				// third southernmost tile
		totalCombatStrengthGrid[rowOffset(oldRow, 2)][colOffset(oldCol, 1)]--;
		if (passableGrid[rowOffset(oldRow, -1)][oldCol] == 1) {									// add combat values if north tile is passable
			combatStrengthGrid[owner][rowOffset(oldRow, -3)][colOffset(oldCol, -1)]--;
			totalCombatStrengthGrid[rowOffset(oldRow, -3)][colOffset(oldCol, -1)]--;
			combatStrengthGrid[owner][rowOffset(oldRow, -3)][oldCol]--;
			totalCombatStrengthGrid[rowOffset(oldRow, -3)][oldCol]--;
			combatStrengthGrid[owner][rowOffset(oldRow, -3)][colOffset(oldCol, 1)]--;
			totalCombatStrengthGrid[rowOffset(oldRow, -3)][colOffset(oldCol, 1)]--;
		}
		if (passableGrid[oldRow][colOffset(oldCol, -1)] == 1) {									// add combat values if west tile is passable
			combatStrengthGrid[owner][rowOffset(oldRow, -1)][colOffset(oldCol, -3)]--;
			totalCombatStrengthGrid[rowOffset(oldRow, -1)][colOffset(oldCol, -3)]--;
			combatStrengthGrid[owner][oldRow][colOffset(oldCol, -3)]--;
			totalCombatStrengthGrid[oldRow][colOffset(oldCol, -3)]--;
			combatStrengthGrid[owner][rowOffset(oldRow, 1)][colOffset(oldCol, -3)]--;
			totalCombatStrengthGrid[rowOffset(oldRow, 1)][colOffset(oldCol, -3)]--;
		}
		if (passableGrid[oldRow][colOffset(oldCol, 1)] == 1) {									// add combat values if east tile is passable
			combatStrengthGrid[owner][rowOffset(oldRow, -1)][colOffset(oldCol, 3)]--;
			totalCombatStrengthGrid[rowOffset(oldRow, -1)][colOffset(oldCol, 3)]--;
			combatStrengthGrid[owner][oldRow][colOffset(oldCol, 3)]--;
			totalCombatStrengthGrid[oldRow][colOffset(oldCol, 3)]--;
			combatStrengthGrid[owner][rowOffset(oldRow, 1)][colOffset(oldCol, 3)]--;
			totalCombatStrengthGrid[rowOffset(oldRow, 1)][colOffset(oldCol, 3)]--;
		}
		if (passableGrid[rowOffset(oldRow, 1)][oldCol] == 1) {									// add combat values if south tile is passable
			combatStrengthGrid[owner][rowOffset(oldRow, 3)][colOffset(oldCol, -1)]--;
			totalCombatStrengthGrid[rowOffset(oldRow, 3)][colOffset(oldCol, -1)]--;
			combatStrengthGrid[owner][rowOffset(oldRow, 3)][oldCol]--;
			totalCombatStrengthGrid[rowOffset(oldRow, 3)][oldCol]--;
			combatStrengthGrid[owner][rowOffset(oldRow, 3)][colOffset(oldCol, 1)]--;
			totalCombatStrengthGrid[rowOffset(oldRow, 3)][colOffset(oldCol, 1)]--;
		}
		if (passableGrid[rowOffset(oldRow, -1)][oldCol] == 1 || passableGrid[oldRow][colOffset(oldCol, -1)] == 1) {
			combatStrengthGrid[owner][rowOffset(oldRow, -2)][colOffset(oldCol, -2)]--;
			totalCombatStrengthGrid[rowOffset(oldRow, -2)][colOffset(oldCol, -2)]--;
		}
		if (passableGrid[rowOffset(oldRow, -1)][oldCol] == 1 || passableGrid[oldRow][colOffset(oldCol, 1)] == 1) {
			combatStrengthGrid[owner][rowOffset(oldRow, -2)][colOffset(oldCol, 2)]--;
			totalCombatStrengthGrid[rowOffset(oldRow, -2)][colOffset(oldCol, 2)]--;
		}
		if (passableGrid[oldRow][colOffset(oldCol, -1)] == 1 || passableGrid[rowOffset(oldRow, 1)][oldCol] == 1) {
			combatStrengthGrid[owner][rowOffset(oldRow, 2)][colOffset(oldCol, -2)]--;
			totalCombatStrengthGrid[rowOffset(oldRow, 2)][colOffset(oldCol, -2)]--;
		}
		if (passableGrid[oldRow][colOffset(oldCol, 1)] == 1 || passableGrid[rowOffset(oldRow, 1)][oldCol] == 1) {
			combatStrengthGrid[owner][rowOffset(oldRow, 2)][colOffset(oldCol, 2)]--;
			totalCombatStrengthGrid[rowOffset(oldRow, 2)][colOffset(oldCol, 2)]--;
		}
		int newRow = newTile.getRow();
		int newCol = newTile.getCol();
		combatStrengthGrid[owner][newRow][newCol]++;											// own tile
		totalCombatStrengthGrid[newRow][newCol]++;
		combatStrengthGrid[owner][rowOffset(newRow, -2)][colOffset(newCol, -1)]++;				// first northernmost tile
		totalCombatStrengthGrid[rowOffset(newRow, -2)][colOffset(newCol, -1)]++;
		combatStrengthGrid[owner][rowOffset(newRow, -2)][newCol]++;								// second northernmost tile
		totalCombatStrengthGrid[rowOffset(newRow, -2)][newCol]++;
		combatStrengthGrid[owner][rowOffset(newRow, -2)][colOffset(newCol, 1)]++;				// third northernmost tile
		totalCombatStrengthGrid[rowOffset(newRow, -2)][colOffset(newCol, 1)]++;
		combatStrengthGrid[owner][rowOffset(newRow, -1)][colOffset(newCol, -2)]++;				// first north tile
		totalCombatStrengthGrid[rowOffset(newRow, -1)][colOffset(newCol, -2)]++;
		combatStrengthGrid[owner][rowOffset(newRow, -1)][colOffset(newCol, -1)]++;				// second north tile
		totalCombatStrengthGrid[rowOffset(newRow, -1)][colOffset(newCol, -1)]++;
		combatStrengthGrid[owner][rowOffset(newRow, -1)][newCol]++;								// third north tile
		totalCombatStrengthGrid[rowOffset(newRow, -1)][newCol]++;
		combatStrengthGrid[owner][rowOffset(newRow, -1)][colOffset(newCol, 1)]++;				// fourth north tile
		totalCombatStrengthGrid[rowOffset(newRow, -1)][colOffset(newCol, 1)]++;
		combatStrengthGrid[owner][rowOffset(newRow, -1)][colOffset(newCol, 2)]++;				// fifth north tile
		totalCombatStrengthGrid[rowOffset(newRow, -1)][colOffset(newCol, 2)]++;
		combatStrengthGrid[owner][newRow][colOffset(newCol, -2)]++;								// first west tile
		totalCombatStrengthGrid[newRow][colOffset(newCol, -2)]++;
		combatStrengthGrid[owner][newRow][colOffset(newCol, -1)]++;								// second west tile
		totalCombatStrengthGrid[newRow][colOffset(newCol, -1)]++;
		combatStrengthGrid[owner][newRow][colOffset(newCol, 1)]++;								// first east tile
		totalCombatStrengthGrid[newRow][colOffset(newCol, 1)]++;
		combatStrengthGrid[owner][newRow][colOffset(newCol, 2)]++;								// second east tile
		totalCombatStrengthGrid[newRow][colOffset(newCol, 2)]++;
		combatStrengthGrid[owner][rowOffset(newRow, 1)][colOffset(newCol, -2)]++;				// first south tile
		totalCombatStrengthGrid[rowOffset(newRow, 1)][colOffset(newCol, -2)]++;
		combatStrengthGrid[owner][rowOffset(newRow, 1)][colOffset(newCol, -1)]++;				// second south tile
		totalCombatStrengthGrid[rowOffset(newRow, 1)][colOffset(newCol, -1)]++;
		combatStrengthGrid[owner][rowOffset(newRow, 1)][newCol]++;								// third south tile
		totalCombatStrengthGrid[rowOffset(newRow, 1)][newCol]++;
		combatStrengthGrid[owner][rowOffset(newRow, 1)][colOffset(newCol, 1)]++;				// fourth south tile
		totalCombatStrengthGrid[rowOffset(newRow, 1)][colOffset(newCol, 1)]++;
		combatStrengthGrid[owner][rowOffset(newRow, 1)][colOffset(newCol, 2)]++;				// fifth south tile
		totalCombatStrengthGrid[rowOffset(newRow, 1)][colOffset(newCol, 2)]++;
		combatStrengthGrid[owner][rowOffset(newRow, 2)][colOffset(newCol, -1)]++;				// first southernmost tile
		totalCombatStrengthGrid[rowOffset(newRow, 2)][colOffset(newCol, -1)]++;
		combatStrengthGrid[owner][rowOffset(newRow, 2)][newCol]++;								// second southernmost tile
		totalCombatStrengthGrid[rowOffset(newRow, 2)][newCol]++;
		combatStrengthGrid[owner][rowOffset(newRow, 2)][colOffset(newCol, 1)]++;				// third southernmost tile
	}
	
	private void fillCombatGrid(int owner, Tile thisTile) {
		int thisRow = thisTile.getRow();
		int thisCol = thisTile.getCol();
		combatStrengthGrid[owner][thisRow][thisCol]++;											// own tile
		totalCombatStrengthGrid[thisRow][thisCol]++;
		combatStrengthGrid[owner][rowOffset(thisRow, -2)][colOffset(thisCol, -1)]++;			// first northernmost tile
		totalCombatStrengthGrid[rowOffset(thisRow, -2)][colOffset(thisCol, -1)]++;
		combatStrengthGrid[owner][rowOffset(thisRow, -2)][thisCol]++;							// second northernmost tile
		totalCombatStrengthGrid[rowOffset(thisRow, -2)][thisCol]++;
		combatStrengthGrid[owner][rowOffset(thisRow, -2)][colOffset(thisCol, 1)]++;				// third northernmost tile
		totalCombatStrengthGrid[rowOffset(thisRow, -2)][colOffset(thisCol, 1)]++;
		combatStrengthGrid[owner][rowOffset(thisRow, -1)][colOffset(thisCol, -2)]++;			// first north tile
		totalCombatStrengthGrid[rowOffset(thisRow, -1)][colOffset(thisCol, -2)]++;
		combatStrengthGrid[owner][rowOffset(thisRow, -1)][colOffset(thisCol, -1)]++;			// second north tile
		totalCombatStrengthGrid[rowOffset(thisRow, -1)][colOffset(thisCol, -1)]++;
		combatStrengthGrid[owner][rowOffset(thisRow, -1)][thisCol]++;							// third north tile
		totalCombatStrengthGrid[rowOffset(thisRow, -1)][thisCol]++;
		combatStrengthGrid[owner][rowOffset(thisRow, -1)][colOffset(thisCol, 1)]++;				// fourth north tile
		totalCombatStrengthGrid[rowOffset(thisRow, -1)][colOffset(thisCol, 1)]++;
		combatStrengthGrid[owner][rowOffset(thisRow, -1)][colOffset(thisCol, 2)]++;				// fifth north tile
		totalCombatStrengthGrid[rowOffset(thisRow, -1)][colOffset(thisCol, 2)]++;
		combatStrengthGrid[owner][thisRow][colOffset(thisCol, -2)]++;							// first west tile
		totalCombatStrengthGrid[thisRow][colOffset(thisCol, -2)]++;
		combatStrengthGrid[owner][thisRow][colOffset(thisCol, -1)]++;							// second west tile
		totalCombatStrengthGrid[thisRow][colOffset(thisCol, -1)]++;
		combatStrengthGrid[owner][thisRow][colOffset(thisCol, 1)]++;							// first east tile
		totalCombatStrengthGrid[thisRow][colOffset(thisCol, 1)]++;
		combatStrengthGrid[owner][thisRow][colOffset(thisCol, 2)]++;							// second east tile
		totalCombatStrengthGrid[thisRow][colOffset(thisCol, 2)]++;
		combatStrengthGrid[owner][rowOffset(thisRow, 1)][colOffset(thisCol, -2)]++;				// first south tile
		totalCombatStrengthGrid[rowOffset(thisRow, 1)][colOffset(thisCol, -2)]++;
		combatStrengthGrid[owner][rowOffset(thisRow, 1)][colOffset(thisCol, -1)]++;				// second south tile
		totalCombatStrengthGrid[rowOffset(thisRow, 1)][colOffset(thisCol, -1)]++;
		combatStrengthGrid[owner][rowOffset(thisRow, 1)][thisCol]++;							// third south tile
		totalCombatStrengthGrid[rowOffset(thisRow, 1)][thisCol]++;
		combatStrengthGrid[owner][rowOffset(thisRow, 1)][colOffset(thisCol, 1)]++;				// fourth south tile
		totalCombatStrengthGrid[rowOffset(thisRow, 1)][colOffset(thisCol, 1)]++;
		combatStrengthGrid[owner][rowOffset(thisRow, 1)][colOffset(thisCol, 2)]++;				// fifth south tile
		totalCombatStrengthGrid[rowOffset(thisRow, 1)][colOffset(thisCol, 2)]++;
		combatStrengthGrid[owner][rowOffset(thisRow, 2)][colOffset(thisCol, -1)]++;				// first southernmost tile
		totalCombatStrengthGrid[rowOffset(thisRow, 2)][colOffset(thisCol, -1)]++;
		combatStrengthGrid[owner][rowOffset(thisRow, 2)][thisCol]++;							// second southernmost tile
		totalCombatStrengthGrid[rowOffset(thisRow, 2)][thisCol]++;
		combatStrengthGrid[owner][rowOffset(thisRow, 2)][colOffset(thisCol, 1)]++;				// third southernmost tile
		totalCombatStrengthGrid[rowOffset(thisRow, 2)][colOffset(thisCol, 1)]++;
		if (passableGrid[rowOffset(thisRow, -1)][thisCol] == 1) {								// add combat values if north tile is passable
			combatStrengthGrid[owner][rowOffset(thisRow, -3)][colOffset(thisCol, -1)]++;
			totalCombatStrengthGrid[rowOffset(thisRow, -3)][colOffset(thisCol, -1)]++;
			combatStrengthGrid[owner][rowOffset(thisRow, -3)][thisCol]++;
			totalCombatStrengthGrid[rowOffset(thisRow, -3)][thisCol]++;
			combatStrengthGrid[owner][rowOffset(thisRow, -3)][colOffset(thisCol, 1)]++;
			totalCombatStrengthGrid[rowOffset(thisRow, -3)][colOffset(thisCol, 1)]++;
		}
		if (passableGrid[thisRow][colOffset(thisCol, -1)] == 1) {								// add combat values if west tile is passable
			combatStrengthGrid[owner][rowOffset(thisRow, -1)][colOffset(thisCol, -3)]++;
			totalCombatStrengthGrid[rowOffset(thisRow, -1)][colOffset(thisCol, -3)]++;
			combatStrengthGrid[owner][thisRow][colOffset(thisCol, -3)]++;
			totalCombatStrengthGrid[thisRow][colOffset(thisCol, -3)]++;
			combatStrengthGrid[owner][rowOffset(thisRow, 1)][colOffset(thisCol, -3)]++;
			totalCombatStrengthGrid[rowOffset(thisRow, 1)][colOffset(thisCol, -3)]++;
		}
		if (passableGrid[thisRow][colOffset(thisCol, 1)] == 1) {								// add combat values if east tile is passable
			combatStrengthGrid[owner][rowOffset(thisRow, -1)][colOffset(thisCol, 3)]++;
			totalCombatStrengthGrid[rowOffset(thisRow, -1)][colOffset(thisCol, 3)]++;
			combatStrengthGrid[owner][thisRow][colOffset(thisCol, 3)]++;
			totalCombatStrengthGrid[thisRow][colOffset(thisCol, 3)]++;
			combatStrengthGrid[owner][rowOffset(thisRow, 1)][colOffset(thisCol, 3)]++;
			totalCombatStrengthGrid[rowOffset(thisRow, 1)][colOffset(thisCol, 3)]++;
		}
		if (passableGrid[rowOffset(thisRow, 1)][thisCol] == 1) {								// add combat values if south tile is passable
			combatStrengthGrid[owner][rowOffset(thisRow, 3)][colOffset(thisCol, -1)]++;
			totalCombatStrengthGrid[rowOffset(thisRow, 3)][colOffset(thisCol, -1)]++;
			combatStrengthGrid[owner][rowOffset(thisRow, 3)][thisCol]++;
			totalCombatStrengthGrid[rowOffset(thisRow, 3)][thisCol]++;
			combatStrengthGrid[owner][rowOffset(thisRow, 3)][colOffset(thisCol, 1)]++;
			totalCombatStrengthGrid[rowOffset(thisRow, 3)][colOffset(thisCol, 1)]++;
		}
		if (passableGrid[rowOffset(thisRow, -1)][thisCol] == 1 || passableGrid[thisRow][colOffset(thisCol, -1)] == 1) {
			combatStrengthGrid[owner][rowOffset(thisRow, -2)][colOffset(thisCol, -2)]++;
			totalCombatStrengthGrid[rowOffset(thisRow, -2)][colOffset(thisCol, -2)]++;
		}
		if (passableGrid[rowOffset(thisRow, -1)][thisCol] == 1 || passableGrid[thisRow][colOffset(thisCol, 1)] == 1) {
			combatStrengthGrid[owner][rowOffset(thisRow, -2)][colOffset(thisCol, 2)]++;
			totalCombatStrengthGrid[rowOffset(thisRow, -2)][colOffset(thisCol, 2)]++;
		}
		if (passableGrid[thisRow][colOffset(thisCol, -1)] == 1 || passableGrid[rowOffset(thisRow, 1)][thisCol] == 1) {
			combatStrengthGrid[owner][rowOffset(thisRow, 2)][colOffset(thisCol, -2)]++;
			totalCombatStrengthGrid[rowOffset(thisRow, 2)][colOffset(thisCol, -2)]++;
		}
		if (passableGrid[thisRow][colOffset(thisCol, 1)] == 1 || passableGrid[rowOffset(thisRow, 1)][thisCol] == 1) {
			combatStrengthGrid[owner][rowOffset(thisRow, 2)][colOffset(thisCol, 2)]++;
			totalCombatStrengthGrid[rowOffset(thisRow, 2)][colOffset(thisCol, 2)]++;
		}
	}
		
	
	// do row offset, taking wraparound map into account
	private int rowOffset(int row, int offset) {
        int result = (row + offset) % rows;
        if (result < 0) {
            result += rows;
        }
		return result;
    }
	
	// do col offset, taking wraparound map into account
	private int colOffset(int col, int offset) {
        int result = (col + offset) % cols;
        if (result < 0) {
            result += cols;
        }
		return result;
    }
	
	// main move function
	private boolean doMoveLocation(Ants ants, Tile antLoc, Tile destLoc) {
		int destLocRow = destLoc.getRow();
		int destLocCol = destLoc.getCol();
		if ((ants.getIlk(destLoc).isUnoccupied() && !orders.containsKey(destLoc)) || ((orders.containsValue(destLoc)) && !orders.containsKey(destLoc))) {
			if (status[0][destLocRow][destLocCol] > 0) {
				List<Aim> directions = ants.getDirections(antLoc, destLoc);
				for (Aim direction : directions) {
					ants.issueOrder(antLoc, direction);
					orders.put(destLoc, antLoc);
				}
			return true;
			}
			return false;
        }
		else {
			return false;
		}
    }
	
	@Override
    public void doTurn() {
		Ants ants = getAnts();
		orders.clear();
		Set<Tile> seenFood = new HashSet<Tile>(ants.getFoodTiles());
		Set<Tile> availableAnts = new HashSet<Tile>(ants.getMyAnts());
		Set<Tile> enemyHills = new HashSet<Tile>(ants.getEnemyHills());
		Set<Tile> enemyAnts = new HashSet<Tile>(ants.getEnemyAnts());
		Set<Tile> deadAnts = new HashSet<Tile>(ants.getDeadAnts());
		Set<Tile> myHills = new HashSet<Tile>(ants.getMyHills());
		Map<Tile, Integer> tacMoveMyAnts = new HashMap<Tile, Integer>();
		int weakness[][][];
		double tempGrid[][];

		rows = ants.getRows();
		cols = ants.getCols();

		// create a grid of all unseen and passable tiles
		// 2 == tile has been seen and is not passable
		// 1 == tile has been seen and is passable
		// 0 == tile has never been seen
		if (passableGrid == null) {
			passableGrid = new int[rows][cols];
		}
		for (int i = 0; i < rows; i++) {
			for (int j = 0; j < cols; j++) {
				if (passableGrid[i][j] == 0) {
				Tile nextTile = new Tile(i, j);
					if (ants.isVisible(nextTile)) { 
						if (ants.getIlk(nextTile).isWater()) {
							passableGrid[i][j] = 2;
						}
						else {
							passableGrid[i][j] = 1;
						}
					}
				}
			}
		}
		
		// create blank scent grid
		if (scentGrid == null) {
			scentGrid = new double[rows][cols];
		}
		
		// create and then update grid of food and ants tiles
		// 6 == my hill is there
		// 5 == dead ant is there
		// 4 == enemy ant is there
		// 3 == enemy hill is there
		// 2 == one of my ants is there
		// 1 == a food is there
		// 0 == nothing is there
		if (foodAntGrid == null) {
			foodAntGrid = new int[rows][cols];
		}
		for (int i = 0; i < rows; i++) {
			for (int j = 0; j < cols; j++) {
				Tile nextTile = new Tile(i, j);
				if (ants.isVisible(nextTile)) {
					if (seenFood.contains(nextTile)) {
						foodAntGrid[i][j] = 1;
					}
					else if (availableAnts.contains(nextTile)) {
						foodAntGrid[i][j] = 2;
					}
					else if (enemyHills.contains(nextTile)) {
						foodAntGrid[i][j] = 3;
					}
					else if (enemyAnts.contains(nextTile)) {
						foodAntGrid[i][j] = 4;
					}
					else if (deadAnts.contains(nextTile)) {
						foodAntGrid[i][j] = 5;
					}
					else if (myHills.contains(nextTile)) {
						foodAntGrid[i][j] = 6;
					}
					else {
						foodAntGrid[i][j] = 0;
					}
				}
				else if (foodAntGrid[i][j] == 2) {
					foodAntGrid[i][j] = 0;
				}
			}
		}

		// create blank enemy grid
		if (enemyGrid == null) {
			enemyGrid = new double[rows][cols];
		}		
		
		// initialize a blank copy of the temporary grid
		// must remember to do this every time before we use the tempGrid
		tempGrid = new double[rows][cols];

		// scentGrid diffusion loop
		for (int h = 0; h < SCENTGRID_DIFFUSION_TIMES; h++) {
			if (ants.getTimeRemaining() > RESERVE_TIME) {
				for (int i = 0; i < rows; i++) {
					for (int j = 0; j < cols; j++) {
						switch (foodAntGrid[i][j]) {
							case 1:
								// tempGrid[i][j] = FOOD_VALUE;
								tempGrid[i][j] = scentGrid[i][j] + FOOD_VALUE;
								break;
							case 2:
								tempGrid[i][j] = 0;
								break;
							default:
								if (passableGrid[i][j] == 0) {
									// tempGrid[i][j] = EXPLORE_VALUE;
									tempGrid[i][j] = scentGrid[i][j] + EXPLORE_VALUE;
								}
								else if (passableGrid[i][j] == 1) {							
									double newScent = 0;
									newScent += scentGrid[rowOffset(i, -1)][j];
									newScent += scentGrid[i][colOffset(j, 1)];
									newScent += scentGrid[rowOffset(i, 1)][j];
									newScent += scentGrid[i][colOffset(j, -1)];
									newScent *= DIFFUSION_RATE;
									Tile newTile = new Tile(i, j);
									if (!ants.isVisible(newTile)) {
										newScent += UNSEEN_BONUS;
									}
										tempGrid[i][j] = newScent;
								}
								else {
									tempGrid[i][j] = 0;
								}
						}
					}
				}
				for (int k = 0; k < rows; k++) {
					for (int l = 0; l < cols; l++) {
						scentGrid[k][l] = tempGrid[k][l];		
					}
				}
			}
			else {
				break;
			}
		}
		
		// create grid of ant combat strength values
		enemyPlayersKnown.addAll(ants.getAllEnemyAntOwners());
		combatStrengthGrid = new int[enemyPlayersKnown.size() + 1][rows][cols];
		totalCombatStrengthGrid = new int[rows][cols];
		for (Tile myAnt : availableAnts) {
			fillCombatGrid(0, myAnt);
			tacMoveMyAnts.put(myAnt, 0);
		}
		for (Tile enemyAnt : enemyAnts) {
			fillCombatGrid(ants.getEnemyAntOwner(enemyAnt), enemyAnt);
		}
		
		// create and fill fighting array for each player
		fighting = new int[enemyPlayersKnown.size() + 1][rows][cols];
		fillFightingGrid(ants, tacMoveMyAnts, enemyAnts);
		status = new int[enemyPlayersKnown.size() + 1][rows][cols];
		fillStatusGrid(tacMoveMyAnts);
		
		// initialize a blank copy of the temporary grid
		// must remember to do this every time before we use the tempGrid
		tempGrid = new double[rows][cols];
		
		// enemyGrid diffusion loop
		for (int h = 0; h < ENEMYGRID_DIFFUSION_TIMES; h++) {
			if (ants.getTimeRemaining() > RESERVE_TIME) {
				for (int i = 0; i < rows; i++) {
					for (int j = 0; j < cols; j++) {
						if (foodAntGrid[i][j] == 3) {
							// tempGrid[i][j] = ENEMY_HILL_VALUE;
							tempGrid[i][j] = enemyGrid[i][j] + ENEMY_HILL_VALUE;
						}
						else if (passableGrid[i][j] == 1) {
							double newScent = 0;
							newScent += enemyGrid[rowOffset(i, -1)][j];
							newScent += enemyGrid[i][colOffset(j, 1)];
							newScent += enemyGrid[rowOffset(i, 1)][j];
							newScent += enemyGrid[i][colOffset(j, -1)];
							newScent *= DIFFUSION_RATE;
							if (foodAntGrid[i][j] == 4) {
								newScent += ENEMY_ANT_BONUS;
							}
							if (foodAntGrid[i][j] == 5) {
								newScent += DEAD_ANT_BONUS;
							}
							tempGrid[i][j] = newScent;
						}
						else {
							tempGrid[i][j] = 0;
						}
					}
				}
				for (int k = 0; k < rows; k++) {
					for (int l = 0; l < cols; l++) {
						enemyGrid[k][l] = tempGrid[k][l];		
					}
				}
			}
			else {
				break;
			}
		}

		Iterator<Tile> myAnts = availableAnts.iterator();
		while (myAnts.hasNext()) {
			Tile myAnt = myAnts.next();
			int antRow = myAnt.getRow();
			int antCol = myAnt.getCol();
			if (status[0][antRow][antCol] == 0) {
				if (foodAntGrid[antRow][antCol] == 6) {
					refillCombatGrid(0, myAnt, myAnt);
					tacMoveMyAnts.put(myAnt, 1);
					fillFightingGrid(ants, tacMoveMyAnts, enemyAnts);
					fillStatusGrid(tacMoveMyAnts);
					myAnts.remove();
					continue;
				}	
				List<Tile> safeTiles = new ArrayList<Tile>();
				Tile northTile = ants.getTile(myAnt, new Tile(-1,0));
				int northTileRow = northTile.getRow();
				int northTileCol = northTile.getCol();
				if (passableGrid[northTileRow][northTileCol] == 1 && status[0][northTileRow][northTileCol] > 0) {
					safeTiles.add(northTile);
				}
				Tile southTile = ants.getTile(myAnt, new Tile(1,0));
				int southTileRow = southTile.getRow();
				int southTileCol = southTile.getCol();
				if (passableGrid[southTileRow][southTileCol] == 1 && status[0][southTileRow][southTileCol] > 0) {
					safeTiles.add(southTile);
				}
				Tile eastTile = ants.getTile(myAnt, new Tile(0,1));
				int eastTileRow = eastTile.getRow();
				int eastTileCol = eastTile.getCol();
				if (passableGrid[eastTileRow][eastTileCol] == 1 && status[0][eastTileRow][eastTileCol] > 0) {
					safeTiles.add(eastTile);
				}
				Tile westTile = ants.getTile(myAnt, new Tile(0,-1));
				int westTileRow = westTile.getRow();
				int westTileCol = westTile.getCol();
				if (passableGrid[westTileRow][westTileCol] == 1 && status[0][westTileRow][westTileCol] > 0) {
					safeTiles.add(westTile);
				}
				Collections.shuffle(safeTiles);
				for (Tile nextDirection : safeTiles) {
					if (doMoveLocation(ants, myAnt, nextDirection)) {
						refillCombatGrid(0, myAnt, nextDirection);
						tacMoveMyAnts.remove(myAnt);
						tacMoveMyAnts.put(nextDirection, 1);
						fillFightingGrid(ants, tacMoveMyAnts, enemyAnts);
						fillStatusGrid(tacMoveMyAnts);
						myAnts.remove();
						break;
					}
				}
			}
			else {
				List<Tile> attackTiles = new ArrayList<Tile>();
				Tile northTile = ants.getTile(myAnt, new Tile(-1,0));
				int northTileRow = northTile.getRow();
				int northTileCol = northTile.getCol();
				if (passableGrid[northTileRow][northTileCol] == 1 && status[0][northTileRow][northTileCol] > 1) {
					attackTiles.add(northTile);
				}
				Tile southTile = ants.getTile(myAnt, new Tile(1,0));
				int southTileRow = southTile.getRow();
				int southTileCol = southTile.getCol();
				if (passableGrid[southTileRow][southTileCol] == 1 && status[0][southTileRow][southTileCol] > 1) {
					attackTiles.add(southTile);
				}
				Tile eastTile = ants.getTile(myAnt, new Tile(0,1));
				int eastTileRow = eastTile.getRow();
				int eastTileCol = eastTile.getCol();
				if (passableGrid[eastTileRow][eastTileCol] == 1 && status[0][eastTileRow][eastTileCol] > 1) {
					attackTiles.add(eastTile);
				}
				Tile westTile = ants.getTile(myAnt, new Tile(0,-1));
				int westTileRow = westTile.getRow();
				int westTileCol = westTile.getCol();
				if (passableGrid[westTileRow][westTileCol] == 1 && status[0][westTileRow][westTileCol] > 1) {
					attackTiles.add(westTile);
				}
				Collections.shuffle(attackTiles);
				for (Tile nextDirection : attackTiles) {
					if (doMoveLocation(ants, myAnt, nextDirection)) {
						refillCombatGrid(0, myAnt, nextDirection);
						tacMoveMyAnts.remove(myAnt);
						tacMoveMyAnts.put(nextDirection, 1);
						fillFightingGrid(ants, tacMoveMyAnts, enemyAnts);
						fillStatusGrid(tacMoveMyAnts);
						myAnts.remove();
						break;
					}
				}
			}
		}
		
		// main ant movement loop
		for (Tile myAnt : availableAnts) {
			int i = myAnt.getRow();
			int j = myAnt.getCol();

			Tile bestDest = myAnt;
			double highestScent = scentGrid[i][j] + enemyGrid[i][j];
			
			Tile destTile = new Tile(rowOffset(i, -1), j);
			if ((ants.getIlk(destTile).isUnoccupied() && !orders.containsKey(destTile)) || ((orders.containsValue(destTile)) && !orders.containsKey(destTile))) {
				double northScent = scentGrid[rowOffset(i, -1)][j] + enemyGrid[rowOffset(i, -1)][j];
				if (northScent > highestScent) {
					highestScent = northScent;
					bestDest= destTile;
				}
			}
			destTile = new Tile(i, colOffset(j, 1));
			if ((ants.getIlk(destTile).isUnoccupied() && !orders.containsKey(destTile)) || ((orders.containsValue(destTile)) && !orders.containsKey(destTile))) {
				double eastScent = scentGrid[i][colOffset(j, 1)] + enemyGrid[i][colOffset(j, 1)];
				if (eastScent > highestScent) {
					highestScent = eastScent;
					bestDest = destTile;
				}
			}
			destTile = new Tile(rowOffset(i, 1), j);
			if ((ants.getIlk(destTile).isUnoccupied() && !orders.containsKey(destTile)) || ((orders.containsValue(destTile)) && !orders.containsKey(destTile))) {
				double southScent = scentGrid[rowOffset(i, 1)][j] + enemyGrid[rowOffset(i, 1)][j];
				if (southScent > highestScent) {
					highestScent = southScent;
					bestDest = destTile;
				}
			}
			destTile = new Tile(i, colOffset(j, -1));
			if ((ants.getIlk(destTile).isUnoccupied() && !orders.containsKey(destTile)) || ((orders.containsValue(destTile)) && !orders.containsKey(destTile))) {
				double westScent = scentGrid[i][colOffset(j, -1)] + enemyGrid[i][colOffset(j, -1)];
				if (westScent > highestScent) {
					highestScent = westScent;
					bestDest = destTile;
				}
			}
			
			if (!myAnt.equals(bestDest)) {
				doMoveLocation(ants, myAnt, bestDest);
			}
			else {
				orders.put(bestDest, bestDest);
			}
		}
	}
}
